package skyrix.portlets;

import java.io.IOException;
import java.net.MalformedURLException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Locale;
import java.util.TimeZone;

import javax.servlet.http.HttpServletRequest;

import skyrix.Appointment;
import skyrix.AppointmentManager;

import com.bea.commerce.ebusiness.discount.util.Log;
import com.bea.p13n.usermgmt.profile.ProfileWrapper;
/**
 * @author buhle 
 *
 * Liest alle Termine aus und gibt die erhaltene Collection von Terminen an
 * showAppointments.jsp weiter.
 */
public class AppointmentOrganizer extends Organizer
{
  public AppointmentOrganizer()
  {
  }

  /**
   * Standardmethode, die automatisch vor Aufruf der JSP-Seite abgearbeitet wird bei Portlets
   * 
   * @see org.apache.jetspeed.modules.actions.portlets.JspPortletAction#buildNormalContext(Portlet, RunData)
   */
  public Hashtable getAppointments(HttpServletRequest request)
  {
    Hashtable returnHash = null;
    AppointmentManager am = null;
    PortletProperties pp = new PortletProperties();
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);
    
    try
    {
      int fault = 0;
      int iLines = 10;
      int actLines = 0;
      
      if (request.getParameter("ap_act_lines") != null)
      {
        actLines = Integer.parseInt(request.getParameter("ap_act_lines"));
        pw.setProperty("skyrix", "ap_act_lines", request.getParameter("ap_act_lines"));
      }
      else
      {
        if (pw.getPropertyAsString("skyrix","ap_act_lines") != null) actLines = Integer.parseInt(pw.getPropertyAsString("skyrix","ap_act_lines"));
      }

      // Wieviel Zeilen angezeigt werden sollen wird ausgelesen
      String str_lines = pw.getPropertyAsString("skyrix","apPerPage"); 
      if (str_lines != null && str_lines.length() > 0)
        iLines = Integer.parseInt(str_lines);
      // Wenn der Wert 0 oder niedriger ist, wird er auf 10 gesetzt.
      if (iLines <1) iLines = 10;

      // Minustage werden ausgelesen
      String days_to_start = "0";
      String days_to_end = "7";
      if (pw.getPropertyAsString("skyrix","apStartDay") != null) days_to_start = pw.getPropertyAsString("skyrix","apStartDay"); 
      if (pw.getPropertyAsString("skyrix","apEndDay") != null) days_to_end = pw.getPropertyAsString("skyrix","apEndDay"); 
      
      int iBegin = Integer.parseInt(days_to_start);
      // Plustage werden ausgelesen
      int iEnd = Integer.parseInt(days_to_end);
      
      // Verbindung aufbauen 
      try
      {
//        Client client = new Client(pp.getHost(), rundata.getUser().getUserName(), rundata.getUser().getPassword());
        Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
        am = new AppointmentManager(client.getClient());
      }
      catch (MalformedURLException ex)
      {
        Log.debug(ex.toString());
        fault = 1;
      }
      catch (IOException ex)
      {
        Log.debug(ex.toString());
        fault = 2;
      }
      
      // Start- und Enddatum werden bestimmt  
      Calendar begin = Calendar.getInstance();
      Calendar end = Calendar.getInstance();
      begin.add(Calendar.DAY_OF_YEAR, iBegin);
      end.add(Calendar.DAY_OF_YEAR, iEnd);
      
      // Alle Termine werden geholt.
      Collection collection = am.getAllAppointments(begin.getTime(), end.getTime());
      ArrayList appointmentList = null;
      String sortBy = "";
      boolean ascending = false;
      
      if (request.getParameter("apAscending") != null) ascending = true;
      if (request.getParameter("apSortBy") != null) sortBy = request.getParameter("apSortBy");
      else 
      {
        ascending = true;
        sortBy = "startdate";
      } 
            
      // Bildet aus der Collection eine ArrayList die nur die aktuell angezeigten Termine enthlt
      boolean next = true;
      boolean prior = true;
      int endLines = 0;
      if (collection != null)  
      {
        if (actLines >= collection.size()) actLines = 0; 
        appointmentList = new ArrayList(collection);
        appointmentList = sortListBy(appointmentList, sortBy, ascending);
        // Nur wenn die anzuzeigenden Zeilen nicht grsser sind als die existierenden Termine
        if (appointmentList.size() > iLines)
        {
          ArrayList dummy = new ArrayList();
          if (actLines+iLines < appointmentList.size())
          {
            endLines = actLines+iLines;
            if (actLines == 0) prior = false;
          }
          else 
          {
            endLines = appointmentList.size();
            next = false;
          }
          // Die ArrayList wird erstellt.
          for (int i=actLines;i<endLines;i++)
          {
            dummy.add(appointmentList.get(i));
          }
          appointmentList = dummy;
        }
        // Es gibt keine vorherigen oder weiteren Termine die angezeigt werden mssten.
        else 
        {
          next = false;
          prior = false;
        } 
      }

      // Es werden die bentigten Daten der JSP-Seite als Attribute in ihr hinterlegt. 
      returnHash = new Hashtable();
      if (appointmentList != null) returnHash.put("appointments", appointmentList);
      returnHash.put("fault", new Integer(fault));
      returnHash.put("skyrix_link", pp.getSkyrixLink());
      returnHash.put("ap_act_lines", new Integer(endLines));
      returnHash.put("lines", new Integer(iLines));
      returnHash.put("hasNext", new Boolean(next));
      returnHash.put("hasPrior", new Boolean(prior));
    }
    catch (Exception e)
    {
      Log.error(e);
    }
    
    return returnHash;
  }
 
  /**Editiert den Termin
   * 
   * @param ap Der zu editierende Termin
   * @param request HttpServletRequest der aufrufenden JSP-Seite     
   */
  public void editAppointment(Appointment ap, Hashtable data, HttpServletRequest request)
  {
    PortletProperties pp = new PortletProperties();
    AppointmentManager am = null;
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

    try
    {
      Locale loc = new Locale("en", "GB");
      DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, loc);
      java.util.Date startdate = new java.util.Date();
      java.util.Date enddate = new java.util.Date();
      
      String strStartdate = (String)data.get("startdate");
      String strEnddate = (String)data.get("enddate");
      startdate = df.parse(strStartdate);
      enddate = df.parse(strEnddate);
      int start_offset = Integer.parseInt((String)data.get("TimeZoneStartOffset"));
      int end_offset = Integer.parseInt((String)data.get("TimeZoneEndOffset"));
      long beginmillis = startdate.getTime() + start_offset;
      long endmillis = enddate.getTime() + end_offset;
      
      Calendar cal_begin = Calendar.getInstance();
      Calendar cal_end = Calendar.getInstance();
      cal_begin.setTimeInMillis(beginmillis);
      cal_end.setTimeInMillis(endmillis);
      cal_begin.setTimeZone(TimeZone.getTimeZone("GMT"));
//      cal_begin.set(Calendar.ZONE_OFFSET, 0);
      cal_end.setTimeZone(TimeZone.getTimeZone("GMT"));
//      cal_end.set(Calendar.ZONE_OFFSET, 0);
      
      ap.setTitle((String)data.get("title"));
      ap.setLocation((String)data.get("location"));
      ap.setComment((String)data.get("comment"));
      ap.setStartDate(cal_begin.getTime());
      ap.setEndDate(cal_end.getTime());

//      Client client = new Client(pp.getHost(), rundata.getUser().getUserName(), rundata.getUser().getPassword());
      Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
      am = new AppointmentManager(client.getClient());
      am.updateAppointment(ap);
//      Log.debug("appointment edited. ap_id = "+ap.getID());
    }
    catch (Exception ex)
    {
      Log.debug(ex.toString());
    }
  }
  
  /**Lscht den Termin
   * 
   * @param alAppointments ArrayList mit zu lschenden Terminen
   * @param request HttpServletRequest der aufrufenden JSP-Seite     
   */
  public void deleteAppointments(ArrayList alAppointments, HttpServletRequest request)
  {
    PortletProperties pp = new PortletProperties();
    AppointmentManager am = null;
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

    try
    {
//      Client client = new Client(pp.getHost(), rundata.getUser().getUserName(), rundata.getUser().getPassword());
      Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
      am = new AppointmentManager(client.getClient());
      Iterator it = alAppointments.iterator();
      while (it.hasNext())
      {
        Appointment ap = (Appointment)it.next();
        am.deleteAppointment(ap);
      }
//      Log.debug("appointment gelscht. ap_id = "+ap.getID());
    }
    catch (Exception ex)
    {
      Log.debug(ex.toString());
    }
  }

  /**Methode zum Anlegen eines neuen Termins
   * 
   * @param rundata
   * @param request HttpServletRequest der aufrufenden JSP-Seite     
   */
  public void createAppointment(HttpServletRequest request)
  {
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

    try
    {
      String title = request.getParameter("title");
      String location = request.getParameter("location");
      String comment = request.getParameter("comment");
      String startdate = request.getParameter("start_date");
      String enddate = request.getParameter("end_date");
      startdate = startdate+" "+request.getParameter("start_hour")+":"+request.getParameter("start_minute");
      enddate = enddate+" "+request.getParameter("end_hour")+":"+request.getParameter("end_minute");

      AppointmentManager am = null;
      PortletProperties pp = new PortletProperties();
    
      Locale loc = new Locale("en", "GB");
      DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT, loc);
      java.util.Date begin = df.parse(startdate);
      java.util.Date end = df.parse(enddate);
      int start_offset = Integer.parseInt(request.getParameter("TimeZoneStartOffset"));
      int end_offset = Integer.parseInt(request.getParameter("TimeZoneEndOffset"));
      long beginmillis = begin.getTime() + start_offset;
      long endmillis = end.getTime() + end_offset;
      
      Calendar cal_begin = Calendar.getInstance();
      Calendar cal_end = Calendar.getInstance();
      cal_begin.setTimeInMillis(beginmillis);
      cal_end.setTimeInMillis(endmillis);
      cal_begin.setTimeZone(TimeZone.getTimeZone("GMT"));
//      cal_begin.set(Calendar.ZONE_OFFSET, 0);
      cal_end.setTimeZone(TimeZone.getTimeZone("GMT"));
//      cal_end.set(Calendar.ZONE_OFFSET, 0);

      Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
      am = new AppointmentManager(client.getClient());
      Appointment ap = new Appointment(null, title, location, comment, cal_begin.getTime(), cal_end.getTime());
      am.insertAppointment(ap);
//      Log.debug("Termin angelegt: "+cal_begin+" - "+cal_end+" / "+title+" / "+comment);
    }
    catch (MalformedURLException ex)
    {
      Log.error(ex);
    }
    catch (IOException ex)
    {
      Log.error(ex);
    }
    catch (Exception ex)
    {
      Log.error(ex);
    }
  }
  
  /**
   * 
   * @param listToSort Zu sortierende ArrayList
   * @param sortBy Nach was sortiert werden soll
   * @param ascending Ob aufsteigend oder absteigend sortiert werden soll
   * @return Sortierte ArrayList aller Termine
   */
  private ArrayList sortListBy(ArrayList listToSort, String sortBy, boolean ascending)
  {
    ArrayList returnList = null;
    if (listToSort != null)
    {
      returnList = new ArrayList();
      Iterator it = listToSort.iterator();
      while (it.hasNext())
      {
        Appointment ap = (Appointment)it.next();
        ArrayList dummyList = new ArrayList();
        if (returnList.size() == 0) returnList.add(ap);
        else
        {
          boolean notFound = true;
          int index = 0;
          Iterator iter = returnList.iterator();
          while (iter.hasNext() && notFound)
          {
            Appointment appointment = (Appointment)iter.next();
            if (ascending)
            {
              if (sortBy.equalsIgnoreCase("title")) 
              {
                if (ap.getTitle().compareToIgnoreCase(appointment.getTitle()) > 0)
                  dummyList.add(appointment);
                else
                {
                  dummyList.add(ap);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("startdate")) 
              {
                if (ap.getStartDate().compareTo(appointment.getStartDate()) > 0)
                  dummyList.add(appointment);
                else
                {
                  dummyList.add(ap);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("location")) 
              {
                if (ap.getLocation().compareToIgnoreCase(appointment.getLocation()) > 0)
                  dummyList.add(appointment);
                else
                {
                  dummyList.add(ap);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
            }
            else
            {
              if (sortBy.equalsIgnoreCase("title")) 
              {
                if (ap.getTitle().compareToIgnoreCase(appointment.getTitle()) < 0)
                  dummyList.add(appointment);
                else
                {
                  dummyList.add(ap);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("startdate")) 
              {
                if (ap.getStartDate().compareTo(appointment.getStartDate()) < 0)
                  dummyList.add(appointment);
                else
                {
                  dummyList.add(ap);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("location")) 
              {
                if (ap.getLocation().compareToIgnoreCase(appointment.getLocation()) < 0)
                  dummyList.add(appointment);
                else
                {
                  dummyList.add(ap);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
            }
            index++;
            if (index == returnList.size() && notFound) dummyList.add(ap);
          }
          returnList = new ArrayList(dummyList);
        }
      }
    }
    return returnList;
  }
}

