package skyrix.portlets;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import skyrix.Job;
import skyrix.JobManager;

import com.bea.commerce.ebusiness.discount.util.Log;
import com.bea.p13n.usermgmt.profile.ProfileWrapper;

/**
 * @author buhle 
 *
 * Liest alle Jobs aus und gibt die erhaltene Collection von Jobs an 
 * showJobs.jsp weiter.
 */
public class JobOrganizer extends Organizer
{

  public JobOrganizer()
  {
  }
	
  /**Holt alle Jobs fr den definierten Zeitraum
   * 
   * @param request HttpServletRequest der aufrufenden JSP-Seite     
   * @return Gibt eine Hashtable mit den Jobs und Parametern zurck
   */
  public Hashtable getJobs(HttpServletRequest request)
	{
    Hashtable returnHash = new Hashtable();
    JobManager jm = null;
    PortletProperties pp = new PortletProperties();
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

		try
		{
      int iLines = 10;
      int actLines = 0;
      
      if (request.getParameter("job_act_lines") != null)
      {
        actLines = Integer.parseInt(request.getParameter("job_act_lines"));
        pw.setProperty("skyrix", "job_act_lines", request.getParameter("job_act_lines"));
      }
      else
      {
        if (pw.getPropertyAsString("skyrix","job_act_lines") != null) actLines = Integer.parseInt(pw.getPropertyAsString("skyrix","job_act_lines"));
      }
      
      // Wieviel Zeilen angezeigt werden sollen wird ausgelesen
      String strLines = pw.getPropertyAsString("skyrix","jobsPerPage");
      if (strLines != null && strLines.length() > 0)
        iLines = Integer.parseInt(strLines);
      // Wenn der Wert 0 oder niedriger ist, wird er auf 10 gesetzt.
      if (iLines <1) iLines = 10;
      
      // Das Startdatum wird ausgelesen
			String strBegin = pw.getPropertyAsString("skyrix","jobsStartDate");
			// Das Enddatum wird ausgelesen
			String strEnd = pw.getPropertyAsString("skyrix","jobsEndDate");
			// Verbindung aufbauen 
  		try
      {
        Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
        jm = new JobManager(client.getClient());
      }
      catch (Exception ex)
      {
        Log.debug(ex.toString());
      }
      
			Locale loc = new Locale("en", "GB");
			DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, loc);
			// Beide Datumsangaben werden versucht in ein Datum zu parsen 
			java.util.Date begin = new java.util.Date();
			java.util.Date end = new java.util.Date();
			
      try
			{
				java.util.Date newBegin = df.parse(strBegin);
				if (newBegin.getTime() > begin.getTime())
					begin = newBegin;
			}
			catch (Exception e){}
			
      try
			{
				java.util.Date newEnd = df.parse(strEnd);
				if (newEnd.getTime() > end.getTime())
					end = newEnd;
			}
			catch (Exception e){}
      
			// Wenn das Enddatum kleiner ist als das Startdatum werden beide gleichgesetzt
      if (end.getTime() <= begin.getTime())
      {
        Calendar cal = Calendar.getInstance();
        cal.setTime(begin);
        cal.add(Calendar.DAY_OF_YEAR, 1);
        end = cal.getTime();
      }  
      
			// Alle Jobs werden geholt.
			Collection collection = jm.getAllJobs(begin, end);
      ArrayList jobList = null;
      
      // Bildet aus der Collection eine ArrayList die nur die aktuell angezeigten Jobs enthlt
      boolean next = true;
      boolean prior = true;
      int endLines = 0;
      if (collection != null)  
			{
        if (actLines >= collection.size()) actLines = 0; 
        jobList = new ArrayList(collection);
        // Nur wenn die anzuzeigenden Zeilen nicht grsser sind als die existierenden Jobs
        if (jobList.size() > iLines)
        {
          ArrayList dummy = new ArrayList();
          if (actLines+iLines < jobList.size())
          {
            endLines = actLines+iLines;
            if (actLines == 0) prior = false;
          }
          else 
          {
            endLines = jobList.size();
            next = false;
          }
          // Die ArrayList wird erstellt.
          for (int i=actLines;i<endLines;i++)
          {
            dummy.add(jobList.get(i));
          }
          jobList = dummy;
        }
        // Es gibt keine vorherigen oder weiteren Jobs die angezeigt werden mssten.
        else 
        {
          next = false;
          prior = false;
        } 
			}
      
      // Es werden die bentigten Daten der JSP-Seite als Attribute in ihr hinterlegt. 
			if (jobList != null) returnHash.put("jobs", jobList);
      returnHash.put("skyrix_link", pp.getSkyrixLink());
      returnHash.put("job_act_lines", new Integer(endLines));
      returnHash.put("lines", new Integer(iLines));
      returnHash.put("hasNext", new Boolean(next));
      returnHash.put("hasPrior", new Boolean(prior));
		}
		catch (Exception e)
		{
			Log.error(e);
		}
    
    return returnHash;
	}
  
  /**Setzt den Status des Jobs auf done 
   * 
   * @param alJobs ArrayList mit Jobs deren Status auf done gesetzt werden soll. 
   * @param rundata 
   */
  public void doneJobs(ArrayList alJobs, HttpServletRequest request)
  {
    PortletProperties pp = new PortletProperties();
    JobManager jm = null;
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

    try
    {
      Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
      jm = new JobManager(client.getClient());
      Iterator it = alJobs.iterator();
      while (it.hasNext())
      {
        Job job = (Job)it.next();
        if (!job.getStatus().equals("done")) jm.doneJob(job);
      }
//      Log.debug("job done. job_id = "+job.getID());
    }
    catch (Exception ex)
    {
      Log.debug(ex.toString());
    }
  }

  /**Archiviert den Job
   * 
   * @param alJobs ArrayList mit zu archivierenden Jobs
   * @param request HttpServletRequest der aufrufenden JSP-Seite     
   */
  public void archiveJobs(ArrayList alJobs, HttpServletRequest request)
  {
    PortletProperties pp = new PortletProperties();
    JobManager jm = null;
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

    try
    {
      Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
      jm = new JobManager(client.getClient());
      Iterator it = alJobs.iterator();
      while (it.hasNext())
      {
        Job job = (Job)it.next();
        jm.archiveJob(job);
//        Log.debug("job archived. job_id = "+job.getID());
      }
    }
    catch (Exception ex)
    {
      Log.debug(ex.toString());
    }
  }

  /**Editiert den Job
   * 
   * @param job Der zu editierende Job
   * @param request HttpServletRequest der aufrufenden JSP-Seite     
   */
  public void editJob(Job job, HttpServletRequest request)
  {
    PortletProperties pp = new PortletProperties();
    JobManager jm = null;
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

    try
    {
      Locale loc = new Locale("en", "GB");
      DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, loc);
      java.util.Date startdate = new java.util.Date();
      java.util.Date enddate = new java.util.Date();
      
      String strStartdate = request.getParameter("jobStartdate");
      String strEnddate = request.getParameter("jobEnddate");
      startdate = df.parse(strStartdate);
      enddate = df.parse(strEnddate);
      
      job.setName(request.getParameter("jobName"));
      job.setCategory(request.getParameter("jobCategory"));
      job.setPriority(request.getParameter("jobPriority"));
      job.setStartDate(startdate);
      job.setEndDate(enddate);

      Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
      jm = new JobManager(client.getClient());
      jm.updateJob(job);
//      Log.debug("job edited. job_id = "+job.getID());
    }
    catch (Exception ex)
    {
      Log.debug(ex.toString());
    }
  }

  /**Methode zum anlegen eines neuen Jobs.
   * 
   * @param request HttpServletRequest der aufrufenden JSP-Seite     
   */
  public void createJob(HttpServletRequest request)
  {
    UserProfileManager upm = new UserProfileManager();
    ProfileWrapper pw = upm.getProfileWrapper(request);

    try
    {
      String name = request.getParameter("name");
      String startdate = request.getParameter("start_date");
      String enddate = request.getParameter("end_date");
      String priority = request.getParameter("priority");
      String category = request.getParameter("category");

      JobManager jm = null;
      PortletProperties pp = new PortletProperties();
    
      Locale loc = new Locale("en", "GB");
      DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, loc);
      java.util.Date begin = df.parse(startdate);
      java.util.Date end = df.parse(enddate);

      Client client = new Client(pp.getHost(), pw.getPropertyAsString("skyrix","userName"), pw.getPropertyAsString("skyrix","userPwd"));
      jm = new JobManager(client.getClient());
      Job job = new Job(null, name, begin, end, null, category, priority);
      jm.insertJob(job);
//      Log.debug("Job angelegt: "+startdate+" - "+enddate+" / "+name);
    }
    catch (Exception ex)
    {
      Log.debug(ex.toString());
    }
  }

  /**
   * 
   * @param listToSort Zu sortierende ArrayList
   * @param sortBy Nach was sortiert werden soll
   * @param ascending Ob aufsteigend oder absteigend sortiert werden soll
   * @return Sortierte ArrayList aller Jobs
   */
   private ArrayList sortListBy(ArrayList listToSort, String sortBy, boolean ascending)
  {
    ArrayList returnList = null;
    if (listToSort != null)
    {
      returnList = new ArrayList();
      Iterator it = listToSort.iterator();
      while (it.hasNext())
      {
        Job jb = (Job)it.next();
        ArrayList dummyList = new ArrayList();
        if (returnList.size() == 0) returnList.add(jb);
        else
        {
          boolean notFound = true;
          int index = 0;
          Iterator iter = returnList.iterator();
          while (iter.hasNext() && notFound)
          {
            Job job = (Job)iter.next();
            if (ascending)
            {
              if (sortBy.equalsIgnoreCase("status")) 
              {
                if (jb.getStatus().compareToIgnoreCase(job.getStatus()) > 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("name")) 
              {
                if (jb.getName().compareToIgnoreCase(job.getName()) > 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("duedate")) 
              {
                if (jb.getEndDate().compareTo(job.getEndDate()) > 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("priority")) 
              {
                if (jb.getPriority().compareToIgnoreCase(job.getPriority()) > 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("category")) 
              {
                if (jb.getCategory().compareToIgnoreCase(job.getCategory()) > 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
            }
            else
            {
              if (sortBy.equalsIgnoreCase("status")) 
              {
                if (jb.getStatus().compareToIgnoreCase(job.getStatus()) < 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("name")) 
              {
                if (jb.getName().compareToIgnoreCase(job.getName()) < 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("duedate")) 
              {
                if (jb.getEndDate().compareTo(job.getEndDate()) < 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("priority")) 
              {
                if (jb.getPriority().compareToIgnoreCase(job.getPriority()) < 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
              if (sortBy.equalsIgnoreCase("category")) 
              {
                if (jb.getCategory().compareToIgnoreCase(job.getCategory()) < 0)
                  dummyList.add(job);
                else
                {
                  dummyList.add(jb);
                  dummyList.addAll(returnList.subList(index, returnList.size()));
                  notFound = false;
                }
              }
            }
            index++;
            if (index == returnList.size() && notFound) dummyList.add(jb);
          }
          returnList = new ArrayList(dummyList);
        }
      }
    }
    return returnList;
  }
}
